@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2009 Ben Cooksley <bcooksley@kde.org>
  *   SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
+ *   SPDX-FileCopyrightText: 2022 ivan tkachenko <me@ratijas.tk>
  *
  *   SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -21,6 +22,7 @@
 #include <QScreen>
 #include <QTimer>
 #include <QVariantList>
+#include <QtGlobal>
 
 #include <KAboutData>
 #include <KActionCollection>
@@ -28,6 +30,8 @@
 #include <KConfigGroup>
 #include <KDesktopFile>
 #include <KFileUtils>
+#include <KIO/JobUiDelegateFactory>
+#include <KIO/OpenUrlJob>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KPluginMetaData>
@@ -59,10 +63,10 @@ SettingsBase::SettingsBase(BaseMode::ApplicationMode mode, QWidget *parent)
     setProperty("_breeze_no_separator", true);
 
     if (m_mode == BaseMode::InfoCenter) {
-        setWindowTitle(i18n("Info Center"));
+        setWindowTitle(i18nd("systemsettings", "Info Center"));
         setWindowIcon(QIcon::fromTheme(QStringLiteral("hwinfo")));
     } else {
-        setWindowTitle(i18n("System Settings"));
+        setWindowTitle(i18nd("systemsettings", "System Settings"));
         setWindowIcon(QIcon::fromTheme(QStringLiteral("preferences-system")));
     }
 
@@ -143,24 +147,41 @@ void SettingsBase::initToolBar()
 {
     // Fill the toolbar with default actions
     // Exit is the very last action
-    quitAction = actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("quit_action"), this, SLOT(close()));
+    quitAction = actionCollection()->addAction(KStandardAction::Quit, QStringLiteral("quit_action"), this, &QWidget::close);
 
     if (m_mode == BaseMode::SystemSettings) {
         switchToIconAction = actionCollection()->addAction(QStringLiteral("switchto_iconview"), this, [this] {
             BaseConfig::setActiveView(QStringLiteral("systemsettings_icon_mode"));
             changePlugin();
         });
-        switchToIconAction->setText(i18n("Switch to Icon View"));
+        switchToIconAction->setText(i18nd("systemsettings", "Switch to Icon View"));
         switchToIconAction->setIcon(QIcon::fromTheme(QStringLiteral("view-list-icons")));
 
         switchToSidebarAction = actionCollection()->addAction(QStringLiteral("switchto_sidebar"), this, [this] {
             BaseConfig::setActiveView(QStringLiteral("systemsettings_sidebar_mode"));
             changePlugin();
         });
-        switchToSidebarAction->setText(i18n("Switch to Sidebar View"));
+        switchToSidebarAction->setText(i18nd("systemsettings", "Switch to Sidebar View"));
         switchToSidebarAction->setIcon(QIcon::fromTheme(QStringLiteral("view-sidetree")));
+
+        highlightChangesAction = actionCollection()->addAction(QStringLiteral("highlight_changes"), this, [this] {
+            if (activeView) {
+                activeView->toggleDefaultsIndicatorsVisibility();
+            }
+        });
+        highlightChangesAction->setCheckable(true);
+        highlightChangesAction->setText(i18nd("systemsettings", "Highlight Changed Settings"));
+        highlightChangesAction->setIcon(QIcon::fromTheme(QStringLiteral("draw-highlight")));
     }
 
+    reportPageSpecificBugAction = actionCollection()->addAction(QStringLiteral("report_bug_in_current_module"), this, [=] {
+        auto job = new KIO::OpenUrlJob(QUrl(activeView->moduleView()->activeModuleMetadata().bugReportUrl()));
+        job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr));
+        job->start();
+    });
+    reportPageSpecificBugAction->setText(i18nd("systemsettings", "Report a Bug in the Current Page…"));
+    reportPageSpecificBugAction->setIcon(QIcon::fromTheme(QStringLiteral("tools-report-bug")));
+
     // Help after it
     initHelpMenu();
 
@@ -171,7 +192,7 @@ void SettingsBase::initToolBar()
     // Finally the search line-edit
     searchAction = new QWidgetAction(this);
     searchAction->setDefaultWidget(searchText);
-    connect(searchAction, SIGNAL(triggered(bool)), searchText, SLOT(setFocus()));
+    connect(searchAction, &QAction::triggered, searchText, QOverload<>::of(&KLineEdit::setFocus));
     actionCollection()->addAction(QStringLiteral("searchText"), searchAction);
     // Initialise the Window
     setupGUI(Save | Create, QString());
@@ -185,11 +206,11 @@ void SettingsBase::initToolBar()
 
 void SettingsBase::initHelpMenu()
 {
-    helpActionMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("help-contents")), i18n("Help"), this);
+    helpActionMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("help-contents")), i18nd("systemsettings", "Help"), this);
     helpActionMenu->setPopupMode(QToolButton::InstantPopup);
     actionCollection()->addAction(QStringLiteral("help_toolbar_menu"), helpActionMenu);
     // Add the custom actions
-    aboutViewAction = actionCollection()->addAction(KStandardAction::AboutApp, QStringLiteral("help_about_view"), this, SLOT(about()));
+    aboutViewAction = actionCollection()->addAction(KStandardAction::AboutApp, QStringLiteral("help_about_view"), this, &SettingsBase::about);
 }
 
 void SettingsBase::initMenuList(MenuItem *parent)
@@ -341,7 +362,9 @@ void SettingsBase::about()
 void SettingsBase::changePlugin()
 {
     if (m_plugins.empty()) { // We should ensure we have a plugin available to choose
-        KMessageBox::error(this, i18n("System Settings was unable to find any views, and hence has nothing to display."), i18n("No views found"));
+        KMessageBox::error(this,
+                           i18nd("systemsettings", "System Settings was unable to find any views, and hence has nothing to display."),
+                           i18nd("systemsettings", "No views found"));
         close();
         return; // Halt now!
     }
@@ -395,13 +418,27 @@ void SettingsBase::changePlugin()
         tooltipManagers << new ToolTipManager(view);
     }
 
-    changeAboutMenu(activeView->aboutData(), aboutViewAction, i18n("About Active View"));
+    if (highlightChangesAction) {
+        highlightChangesAction->setChecked(activeView->defaultsIndicatorsVisible());
+    }
+
+    changeAboutMenu(activeView->aboutData(), aboutViewAction, i18nd("systemsettings", "About Active View"));
     viewChange(false);
 
     stackedWidget->setCurrentWidget(activeView->mainWidget());
     updateViewActions();
 
     activeView->giveFocus();
+
+    // Update visibility of the "report a bug on this page" and "report bug in general"
+    // actions based on whether the current page has a bug report URL set
+    auto reportGeneralBugAction = actionCollection()->action(QStringLiteral("help_report_bug"));
+    reportGeneralBugAction->setVisible(false);
+    auto moduleView = activeView->moduleView();
+    connect(moduleView, &ModuleView::moduleChanged, this, [=] {
+        reportPageSpecificBugAction->setVisible(!moduleView->activeModuleMetadata().bugReportUrl().isEmpty());
+        reportGeneralBugAction->setVisible(!reportPageSpecificBugAction->isVisible());
+    });
 }
 
 void SettingsBase::viewChange(bool state)
@@ -456,7 +493,7 @@ void SettingsBase::changeAboutMenu(const KAboutData *menuAbout, QAction *menuIte
     }
 
     if (menuAbout) {
-        menuItem->setText(i18n("About %1", menuAbout->displayName()));
+        menuItem->setText(i18nd("systemsettings", "About %1", menuAbout->displayName()));
         menuItem->setIcon(QGuiApplication::windowIcon());
         menuItem->setEnabled(true);
     } else {
