@@ -1,5 +1,6 @@
 /*
  * SPDX-FileCopyrightText: 2009 Ben Cooksley <bcooksley@kde.org>
+ * SPDX-FileCopyrightText: 2022 ivan tkachenko <me@ratijas.tk>
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -11,7 +12,6 @@
 #include "MenuModel.h"
 #include "MenuProxyModel.h"
 #include "ModuleView.h"
-#include "ToolTips/tooltipmanager.h"
 #include "kcmmetadatahelpers.h"
 
 #include <QGuiApplication>
@@ -35,6 +35,7 @@
 #include <QMenu>
 #include <QQmlContext>
 #include <QQmlEngine>
+#include <QQuickItem>
 #include <QQuickWidget>
 #include <QStandardItemModel>
 #include <QStandardPaths>
@@ -117,113 +118,6 @@ void SubcategoryModel::loadParentCategoryModule()
     }
 }
 
-class MostUsedModel : public QSortFilterProxyModel
-{
-public:
-    explicit MostUsedModel(QObject *parent = nullptr)
-        : QSortFilterProxyModel(parent)
-    {
-        sort(0, Qt::DescendingOrder);
-        setSortRole(ResultModel::ScoreRole);
-        setDynamicSortFilter(true);
-        // prepare default items
-        m_defaultModel = new QStandardItemModel(this);
-
-        KService::Ptr service = KService::serviceByDesktopName(qGuiApp->desktopFileName());
-        if (service) {
-            const auto actions = service->actions();
-            for (const KServiceAction &action : actions) {
-                QStandardItem *item = new QStandardItem();
-                item->setData(QUrl(QStringLiteral("kcm:%1.desktop").arg(action.name())), ResultModel::ResourceRole);
-                m_defaultModel->appendRow(item);
-            }
-        } else {
-            qCritical() << "Failed to find desktop file for" << qGuiApp->desktopFileName();
-        }
-    }
-
-    void setResultModel(ResultModel *model)
-    {
-        if (m_resultModel == model) {
-            return;
-        }
-
-        auto updateModel = [this]() {
-            if (m_resultModel->rowCount() >= 5) {
-                setSourceModel(m_resultModel);
-            } else {
-                setSourceModel(m_defaultModel);
-            }
-        };
-
-        m_resultModel = model;
-
-        connect(m_resultModel, &QAbstractItemModel::rowsInserted, this, updateModel);
-        connect(m_resultModel, &QAbstractItemModel::rowsRemoved, this, updateModel);
-
-        updateModel();
-    }
-
-    QHash<int, QByteArray> roleNames() const override
-    {
-        QHash<int, QByteArray> roleNames;
-        roleNames.insert(Qt::DisplayRole, "display");
-        roleNames.insert(Qt::DecorationRole, "decoration");
-        roleNames.insert(ResultModel::ScoreRole, "score");
-        return roleNames;
-    }
-
-    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override
-    {
-        const QString desktopName = sourceModel()->index(source_row, 0, source_parent).data(ResultModel::ResourceRole).toUrl().path();
-        KService::Ptr service = KService::serviceByStorageId(desktopName);
-        return service;
-    }
-
-    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override
-    {
-        MenuItem *mi;
-        const QString desktopName = QSortFilterProxyModel::data(index, ResultModel::ResourceRole).toUrl().path();
-
-        if (m_menuItems.contains(desktopName)) {
-            mi = m_menuItems.value(desktopName);
-        } else {
-            mi = new MenuItem(false, nullptr);
-            const_cast<MostUsedModel *>(this)->m_menuItems.insert(desktopName, mi);
-
-            KService::Ptr service = KService::serviceByStorageId(desktopName);
-
-            if (!service || !service->isValid()) {
-                qWarning() << desktopName;
-                m_resultModel->forgetResource(QStringLiteral("kcm:") % desktopName);
-                return QVariant();
-            }
-            QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kservices5/") + service->entryPath());
-            mi->setMetaData(KPluginMetaData::fromDesktopFile(path));
-        }
-
-        switch (role) {
-        case Qt::UserRole:
-            return QVariant::fromValue(mi);
-        case Qt::DisplayRole:
-            return mi->name();
-        case Qt::DecorationRole:
-            return mi->iconName();
-        case ResultModel::ScoreRole:
-            return QSortFilterProxyModel::data(index, ResultModel::ScoreRole).toInt();
-        default:
-            return QVariant();
-        }
-    }
-
-private:
-    QHash<QString, MenuItem *> m_menuItems;
-    // Model when there is nothing from kactivities-stat
-    QStandardItemModel *m_defaultModel;
-    // Model fed by kactivities-stats
-    ResultModel *m_resultModel;
-};
-
 class SidebarMode::Private
 {
 public:
@@ -241,12 +135,9 @@ public:
         delete aboutIcon;
     }
 
-    ToolTipManager *toolTipManager = nullptr;
-    ToolTipManager *mostUsedToolTipManager = nullptr;
     QQuickWidget *quickWidget = nullptr;
     KPackage::Package package;
     SubcategoryModel *subCategoryModel = nullptr;
-    MostUsedModel *mostUsedModel = nullptr;
     FocusHackWidget *mainWidget = nullptr;
     QQuickWidget *placeHolderWidget = nullptr;
     QHBoxLayout *mainLayout = nullptr;
@@ -333,11 +224,6 @@ QAbstractItemModel *SidebarMode::subCategoryModel() const
     return d->subCategoryModel;
 }
 
-QAbstractItemModel *SidebarMode::mostUsedModel() const
-{
-    return d->mostUsedModel;
-}
-
 QList<QAbstractItemView *> SidebarMode::views() const
 {
     QList<QAbstractItemView *> list;
@@ -366,8 +252,6 @@ void SidebarMode::initEvent()
     d->searchModel->setFilterHighlightsEntries(false);
     d->searchModel->setSourceModel(d->flatModel);
 
-    d->mostUsedModel = new MostUsedModel(this);
-
     d->subCategoryModel = new SubcategoryModel(d->categorizedModel, this);
     d->mainWidget = new FocusHackWidget();
     d->mainWidget->installEventFilter(this);
@@ -408,31 +292,6 @@ QString SidebarMode::actionIconName(const QString &name) const
     return QString();
 }
 
-void SidebarMode::requestToolTip(const QModelIndex &index, const QRectF &rect)
-{
-    if (index.model()) {
-        d->toolTipManager->setModel(index.model());
-        d->toolTipManager->requestToolTip(index, rect.toRect());
-    }
-}
-
-void SidebarMode::requestMostUsedToolTip(int index, const QRectF &rect)
-{
-    d->mostUsedToolTipManager->requestToolTip(d->mostUsedModel->index(index, 0), rect.toRect());
-}
-
-void SidebarMode::hideToolTip()
-{
-    if (d->toolTipManager) {
-        d->toolTipManager->hideToolTip();
-    }
-}
-
-void SidebarMode::hideMostUsedToolTip()
-{
-    d->mostUsedToolTipManager->hideToolTip();
-}
-
 void SidebarMode::showActionMenu(const QPoint &position)
 {
     QMenu *menu = new QMenu();
@@ -442,17 +301,23 @@ void SidebarMode::showActionMenu(const QPoint &position)
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
     const QStringList actionList{QStringLiteral("switchto_iconview"),
+                                 QStringLiteral("highlight_changes"),
                                  QStringLiteral("configure"),
-                                 QStringLiteral("help_contents"),
+                                 QStringLiteral("report_bug_in_current_module"),
                                  QStringLiteral("help_report_bug"),
+                                 QStringLiteral("help_contents"),
                                  QStringLiteral("help_about_app"),
                                  QStringLiteral("help_about_kde")};
 
     for (const QString &actionName : actionList) {
-        menu->addAction(d->collection->action(actionName));
+        QAction *action = d->collection->action(actionName);
+        if (action) {
+            menu->addAction(action);
+        }
     }
 
     menu->popup(position);
+    menu->windowHandle()->setTransientParent(d->quickWidget->window()->windowHandle());
     d->setActionMenuVisible(this, true);
 }
 
@@ -729,6 +594,7 @@ void SidebarMode::refreshDefaults()
         }
     }
 }
+
 void SidebarMode::toggleDefaultsIndicatorsVisibility()
 {
     d->m_defaultsIndicatorsVisible = !d->m_defaultsIndicatorsVisible;
@@ -837,8 +703,6 @@ void SidebarMode::initWidget()
     d->mainLayout->addWidget(d->moduleView);
     Q_EMIT changeToolBarItems(BaseMode::NoItems);
 
-    d->toolTipManager = new ToolTipManager(d->categorizedModel, d->quickWidget, ToolTipManager::ToolTipPosition::Right);
-
     if (!startupModule().isEmpty()) {
         initPlaceHolderWidget();
         d->placeHolderWidget->show();
@@ -873,9 +737,6 @@ void SidebarMode::initPlaceHolderWidget()
     d->placeHolderWidget->installEventFilter(this);
 
     d->mainLayout->addWidget(d->placeHolderWidget);
-
-    d->mostUsedToolTipManager = new ToolTipManager(d->mostUsedModel, d->placeHolderWidget, ToolTipManager::ToolTipPosition::BottomCenter);
-    d->mostUsedModel->setResultModel(new ResultModel(AllResources | Agent(QStringLiteral("org.kde.systemsettings")) | HighScoredFirst | Limit(5), this));
 }
 
 void SidebarMode::reloadStartupModule()
